# ------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# -------------------------------------------------------------------------
import time
import datetime
import random
from typing import (
    Any,
    Dict,
    Mapping,
    Optional,
    Tuple,
)
from ._constants import (
    DEFAULT_STARTUP_TIMEOUT,
    MAX_STARTUP_BACKOFF_DURATION,
    MIN_STARTUP_EXPONENTIAL_BACKOFF_DURATION,
    JITTER_RATIO,
    STARTUP_BACKOFF_INTERVALS,
)


JSON = Mapping[str, Any]

min_uptime = 5


def delay_failure(start_time: datetime.datetime) -> None:
    """
    We want to make sure we are up a minimum amount of time before we kill the process.
    Otherwise, we could get stuck in a quick restart loop.

    :param start_time: The time when the process started.
    :type start_time: datetime.datetime
    """
    min_time = datetime.timedelta(seconds=min_uptime)
    current_time = datetime.datetime.now()
    if current_time - start_time < min_time:
        time.sleep((min_time - (current_time - start_time)).total_seconds())


def get_startup_backoff(elapsed_seconds: float, attempts: int) -> Tuple[float, bool]:
    """
    Get a backoff duration based on elapsed startup time.

    :param elapsed_seconds: The time elapsed since startup began, in seconds.
    :type elapsed_seconds: float
    :param attempts: The number of retry attempts made (1-based).
    :type attempts: int
    :return: A tuple where the first element is the backoff duration in seconds,
             and the second element indicates if the fixed backoff window has been exceeded.
    :rtype: Tuple[float, bool]
    """
    for threshold, backoff in STARTUP_BACKOFF_INTERVALS:
        if elapsed_seconds < threshold:
            return backoff, False
    return _calculate_backoff_duration(attempts), True


def process_load_parameters(*args, **kwargs: Any) -> Dict[str, Any]:
    """
    Process and validate all load function parameters in one place.
    This consolidates the most obviously duplicated logic from both sync and async load functions.

    :param args: Positional arguments, either endpoint and credential, or connection string.
    :type args: Any
    :return: Dictionary containing processed parameters
    :rtype: Dict[str, Any]
    """
    endpoint: Optional[str] = kwargs.pop("endpoint", None)
    credential = kwargs.pop("credential", None)
    connection_string: Optional[str] = kwargs.pop("connection_string", None)
    start_time = datetime.datetime.now()

    # Handle positional arguments
    if len(args) > 2:
        raise TypeError(
            "Unexpected positional parameters. Please pass either endpoint and credential, or a connection string."
        )
    if len(args) == 1:
        if endpoint is not None:
            raise TypeError("Received multiple values for parameter 'endpoint'.")
        endpoint = args[0]
    elif len(args) == 2:
        if credential is not None:
            raise TypeError("Received multiple values for parameter 'credential'.")
        endpoint, credential = args

    # Validate endpoint/credential vs connection_string
    if (endpoint or credential) and connection_string:
        raise ValueError("Please pass either endpoint and credential, or a connection string.")

    # Process Key Vault options in one place
    key_vault_options = kwargs.pop("key_vault_options", None)
    if key_vault_options:
        if "keyvault_credential" in kwargs or "secret_resolver" in kwargs or "keyvault_client_configs" in kwargs:
            raise ValueError(
                "Key Vault configurations should only be set by either the key_vault_options or kwargs not both."
            )
        kwargs["keyvault_credential"] = key_vault_options.credential
        kwargs["secret_resolver"] = key_vault_options.secret_resolver
        kwargs["keyvault_client_configs"] = key_vault_options.client_configs

    if kwargs.get("keyvault_credential") is not None and kwargs.get("secret_resolver") is not None:
        raise ValueError("A keyvault credential and secret resolver can't both be configured.")

    # Determine Key Vault usage
    uses_key_vault = (
        "keyvault_credential" in kwargs
        or "keyvault_client_configs" in kwargs
        or "secret_resolver" in kwargs
        or kwargs.get("uses_key_vault", False)
    )

    # Get startup timeout
    startup_timeout = kwargs.pop("startup_timeout", DEFAULT_STARTUP_TIMEOUT)
    if startup_timeout < 0:
        raise ValueError("Startup timeout must be greater than or equal to 0 seconds.")

    return {
        "endpoint": endpoint,
        "credential": credential,
        "connection_string": connection_string,
        "uses_key_vault": uses_key_vault,
        "start_time": start_time,
        "startup_timeout": startup_timeout,
        "kwargs": kwargs,
    }


def sdk_allowed_kwargs(kwargs: Dict[str, Any]) -> Dict[str, Any]:
    allowed_kwargs = [
        "audience",
        "headers",
        "request_id",
        "user_agent",
        "logging_enable",
        "logger",
        "response_encoding",
        "raw_request_hook",
        "raw_response_hook",
        "network_span_namer",
        "tracing_attributes",
        "permit_redirects",
        "redirect_max",
        "retry_total",
        "retry_connect",
        "retry_read",
        "retry_status",
        "retry_backoff_factor",
        "retry_backoff_max",
        "retry_mode",
        "timeout",
        "connection_timeout",
        "read_timeout",
        "connection_verify",
        "connection_cert",
        "proxies",
        "cookies",
        "connection_data_block_size",
    ]
    return {k: v for k, v in kwargs.items() if k in allowed_kwargs}


def _jitter(duration: float, ratio: float = JITTER_RATIO) -> float:
    """
    Apply jitter to a duration value.

    :param duration: The base duration in seconds.
    :type duration: float
    :param ratio: The jitter ratio (0 to 1). Default is 0.25 (25% jitter means +/- 25% variation).
    :type ratio: float
    :return: The jittered duration in seconds.
    :rtype: float
    """
    if ratio < 0 or ratio > 1:
        raise ValueError("Jitter ratio must be between 0 and 1.")
    if ratio == 0:
        return duration
    jitter = ratio * (random.random() * 2 - 1)
    return duration * (1 + jitter)


def _calculate_backoff_duration(attempts: int) -> float:
    """
    Calculate the jittered exponential backoff duration.

    :param attempts: The number of retry attempts made, using a 0-based counter.
    :type attempts: int
    :return: The calculated backoff duration with jitter applied.
    :rtype: float
    """
    attempts += 1
    if attempts < 1:
        raise ValueError("Number of attempts must be at least 1.")

    if attempts == 1:
        return MIN_STARTUP_EXPONENTIAL_BACKOFF_DURATION

    # Calculate exponential backoff: min * 2^(attempts-1)
    # Cap the shift amount to prevent overflow
    safe_shift = min(attempts - 1, 63)
    calculated = MIN_STARTUP_EXPONENTIAL_BACKOFF_DURATION * (1 << safe_shift)

    # Cap at max duration
    if calculated > MAX_STARTUP_BACKOFF_DURATION or calculated <= 0:  # Check for overflow
        calculated = MAX_STARTUP_BACKOFF_DURATION

    return _jitter(calculated, JITTER_RATIO)



# pylint: disable=line-too-long,useless-suppression
# coding=utf-8
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
FILE: sample_delete_deployment_from_resources_async.py
DESCRIPTION:
    This sample demonstrates how to delete a deployment from specific resources in a Conversation Authoring project (async).

USAGE:
    python sample_delete_deployment_from_resources_async.py

REQUIRED ENV VARS (for AAD / DefaultAzureCredential):
    AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
    AZURE_CLIENT_ID
    AZURE_TENANT_ID
    AZURE_CLIENT_SECRET

NOTE:
    If you want to use AzureKeyCredential instead, set:
      - AZURE_CONVERSATIONS_AUTHORING_ENDPOINT
      - AZURE_CONVERSATIONS_AUTHORING_KEY

OPTIONAL ENV VARS:
    PROJECT_NAME         # defaults to "<project-name>"
    RESOURCE_ID          # defaults to "<azure-resource-id>"
"""

# [START conversation_authoring_delete_deployment_from_resources_async]
import os
import asyncio
from azure.identity.aio import DefaultAzureCredential
from azure.core.exceptions import HttpResponseError
from azure.ai.language.conversations.authoring.aio import ConversationAuthoringClient
from azure.ai.language.conversations.authoring.models import ProjectResourceIds


async def sample_delete_deployment_async():
    # settings
    endpoint = os.environ["AZURE_CONVERSATIONS_AUTHORING_ENDPOINT"]

    project_name = os.environ.get("PROJECT_NAME", "<project-name>")
    resource_id = os.environ.get("RESOURCE_ID", "<azure-resource-id>")

    deployment_name = "test-deployment"

    credential = DefaultAzureCredential()
    async with ConversationAuthoringClient(endpoint, credential=credential) as client:
        project_client = client.get_project_client(project_name)

        delete_body = ProjectResourceIds(azure_resource_ids=[resource_id])

        # start delete-from-resources (async long-running operation)
        poller = await project_client.deployment.begin_delete_deployment_from_resources(
            deployment_name=deployment_name,
            body=delete_body,
        )

        try:
            await poller.result()
            print("Delete completed.")
            print(f"done: {poller.done()}")
            print(f"status: {poller.status()}")
        except HttpResponseError as e:
            print(f"Operation failed: {e.message}")
            print(e.error)


# [END conversation_authoring_delete_deployment_from_resources_async]


async def main():
    await sample_delete_deployment_async()


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.run_until_complete(main())



import argparse
import logging

from ci_tools.parsing import ParsedSetup
from ci_tools.variables import in_ci
from ci_tools.environment_exclusions import is_check_enabled


def entrypoint() -> None:
    parser = argparse.ArgumentParser(
        description="""This utility checks the keywords of a targeted python package. If the keyword 'azure sdk' is not present, error.""",
    )

    parser.add_argument(
        "-t",
        "--target",
        dest="target",
        help=("Directory containing a setup.py"),
    )

    args = parser.parse_args()

    pkg_details = ParsedSetup.from_path(args.target)

    if in_ci():
        if not is_check_enabled(args.target, "verify_keywords"):
            logging.info(f"Package {pkg_details.name} opts-out of keyword verification check.")
            exit(0)

    if "azure sdk" not in pkg_details.keywords:
        print(
            f"Keyword 'azure sdk' not present in keywords for {pkg_details.name}. Before attempting publishing, ensure that package {pkg_details.name} has keyword 'azure sdk' present in the keyword array."
        )
        exit(1)
    else:
        exit(0)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from collections.abc import MutableMapping
from typing import Any, Callable, Optional, TypeVar
from xml.etree import ElementTree as ET

from azure.core import PipelineClient
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.paging import ItemPaged
from azure.core.pipeline import PipelineResponse
from azure.core.rest import HttpRequest, HttpResponse
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict

from ... import models as _models2
from ..._configuration import PageableClientConfiguration
from ..._utils.model_base import _deserialize
from ..._utils.serialization import Deserializer, Serializer

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_xml_pagination_list_with_continuation_request(  # pylint: disable=name-too-long
    *, marker: Optional[str] = None, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    accept = _headers.pop("Accept", "application/xml")

    # Construct URL
    _url = "/payload/pageable/xml/list-with-continuation"

    # Construct parameters
    if marker is not None:
        _params["marker"] = _SERIALIZER.query("marker", marker, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_xml_pagination_list_with_next_link_request(**kwargs: Any) -> HttpRequest:  # pylint: disable=name-too-long
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})

    accept = _headers.pop("Accept", "application/xml")

    # Construct URL
    _url = "/payload/pageable/xml/list-with-next-link"

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, headers=_headers, **kwargs)


class XmlPaginationOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~payload.pageable.PageableClient`'s
        :attr:`xml_pagination` attribute.
    """

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: PipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: PageableClientConfiguration = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list_with_continuation(self, **kwargs: Any) -> ItemPaged["_models2.XmlPet"]:
        """list_with_continuation.

        :return: An iterator like instance of XmlPet
        :rtype: ~azure.core.paging.ItemPaged[~payload.pageable.models.XmlPet]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[list[_models2.XmlPet]] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(_continuation_token=None):

            _request = build_xml_pagination_list_with_continuation_request(
                marker=_continuation_token,
                headers=_headers,
                params=_params,
            )
            path_format_arguments = {
                "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
            }
            _request.url = self._client.format_url(_request.url, **path_format_arguments)
            return _request

        def extract_data(pipeline_response):
            deserialized = ET.fromstring(pipeline_response.http_response.text())
            list_of_elem = _deserialize(
                list[_models2.XmlPet],
                deserialized.find("Pets"),
            )
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            _cont_token_elem = deserialized.find("NextMarker")
            return _cont_token_elem.text if _cont_token_elem is not None else None, iter(list_of_elem)

        def get_next(_continuation_token=None):
            _request = prepare_request(_continuation_token)

            _stream = False
            pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response)

            return pipeline_response

        return ItemPaged(get_next, extract_data)

    @distributed_trace
    def list_with_next_link(self, **kwargs: Any) -> ItemPaged["_models2.XmlPet"]:
        """list_with_next_link.

        :return: An iterator like instance of XmlPet
        :rtype: ~azure.core.paging.ItemPaged[~payload.pageable.models.XmlPet]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[list[_models2.XmlPet]] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_xml_pagination_list_with_next_link_request(
                    headers=_headers,
                    params=_params,
                )
                path_format_arguments = {
                    "endpoint": self._serialize.url(
                        "self._config.endpoint", self._config.endpoint, "str", skip_quote=True
                    ),
                }
                _request.url = self._client.format_url(_request.url, **path_format_arguments)

            else:
                _request = HttpRequest("GET", next_link, headers=_headers)
                path_format_arguments = {
                    "endpoint": self._serialize.url(
                        "self._config.endpoint", self._config.endpoint, "str", skip_quote=True
                    ),
                }
                _request.url = self._client.format_url(_request.url, **path_format_arguments)

            return _request

        def extract_data(pipeline_response):
            deserialized = ET.fromstring(pipeline_response.http_response.text())
            list_of_elem = _deserialize(
                list[_models2.XmlPet],
                deserialized.find("Pets"),
            )
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            _cont_token_elem = deserialized.find("NextLink")
            return _cont_token_elem.text if _cont_token_elem is not None else None, iter(list_of_elem)

        def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response)

            return pipeline_response

        return ItemPaged(get_next, extract_data)



import argparse

import importlib.metadata as importlib_metadata

from packaging.requirements import Requirement

try:
    # pip < 20
    from pip._internal.req import parse_requirements
    from pip._internal.download import PipSession
except:
    # pip >= 20
    from pip._internal.req import parse_requirements
    from pip._internal.network.session import PipSession


def combine_requirements(requirements):
    name = requirements[0].name  # packaging.requirements.Requirement uses 'name' instead of 'project_name'
    specs = []
    for req in requirements:
        if len(req.specifier) == 0:  # packaging.requirements.Requirement uses 'specifier' instead of 'specs'
            continue

        # Convert specifier to the expected format
        specs.extend([str(spec) for spec in req.specifier])

    return name + ",".join(specs)


def get_dependencies(packages):
    requirements = []
    for package in packages:
        try:
            # Get the distribution for this package
            package_info = importlib_metadata.distribution(package)

            # Get requirements and process them like pkg_resources did
            if package_info.requires:
                for req_str in package_info.requires:
                    # Parse the requirement string
                    req = Requirement(req_str)

                    # Apply the same filtering as the original code:
                    # include requirements where marker is None or evaluates to True
                    if req.marker is None or req.marker.evaluate():
                        requirements.append(req)
        except importlib_metadata.PackageNotFoundError:
            # Package not found, skip it (similar to how pkg_resources would handle missing packages)
            continue
        except Exception:
            # Skip packages that can't be processed
            continue

    return requirements


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="List dependencies for a given requirements.txt file")

    parser.add_argument(
        "-r",
        "--requirements",
        dest="requirements_file",
        help="File containing list of packages for which to find dependencies",
        required=True,
    )

    args = parser.parse_args()
    # Get package names from requirements.txt
    requirements = parse_requirements(args.requirements_file, session=PipSession())

    # Handle different pip versions - extract package names
    package_names = []
    for item in requirements:
        if hasattr(item, "requirement"):
            # Parse the requirement string to get the name
            req_str = item.requirement
            if isinstance(req_str, str):
                # Parse the requirement string using packaging.requirements
                try:
                    req = Requirement(req_str)
                    package_names.append(req.name)
                except Exception:
                    # If parsing fails, try to extract name directly
                    name = req_str.split("==")[0].split(">=")[0].split("<=")[0].split(">")[0].split("<")[0].strip()
                    package_names.append(name)
            else:
                package_names.append(req_str.name)
        elif hasattr(item, "req"):
            # Older pip versions
            package_names.append(item.req.name)
        else:
            # Try to get name from the object directly
            try:
                package_names.append(item.name)
            except AttributeError:
                # Skip items we can't parse
                continue

    dependencies = get_dependencies(package_names)

    # It may be the case that packages have multiple sets of dependency
    # requirements, for example:
    # Package A requires Foo>=1.0.0,<2.0.0
    # Package B requires Foo>=1.0.0,<1.2.3
    # This combines all required versions into one string for pip to resolve
    # Output: Foo>=1.0.0,<2.0.0,>=1.0.0,<1.2.3
    # Pip parses this value using the Requirement object (https://setuptools.readthedocs.io/en/latest/pkg_resources.html#requirement-objects)
    # According to https://packaging.python.org/glossary/#term-requirement-specifier
    grouped_dependencies = {}
    for dep in dependencies:
        # Use 'name' instead of 'key' for packaging.requirements.Requirement
        dep_name = dep.name
        if dep_name in grouped_dependencies:
            grouped_dependencies[dep_name].append(dep)
        else:
            grouped_dependencies[dep_name] = [dep]

    final_dependencies = [combine_requirements(r) for r in grouped_dependencies.values()]

    print("\n".join(final_dependencies))



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
# pylint: disable=wrong-import-position

from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ._patch import *  # pylint: disable=unused-wildcard-import


from ._models import (  # type: ignore
    AttestationProvider,
    AttestationProviderListResult,
    AttestationServiceCreationParams,
    AttestationServiceCreationSpecificParams,
    AttestationServicePatchParams,
    AttestationServicePatchSpecificParams,
    CloudError,
    CloudErrorBody,
    JSONWebKey,
    JSONWebKeySet,
    LogSpecification,
    OperationProperties,
    OperationsDefinition,
    OperationsDisplayDefinition,
    PrivateEndpoint,
    PrivateEndpointConnection,
    PrivateEndpointConnectionProperties,
    PrivateLinkResource,
    PrivateLinkResourceListResult,
    PrivateLinkResourceProperties,
    PrivateLinkServiceConnectionState,
    Resource,
    ServiceSpecification,
    StatusResult,
    SystemData,
    TrackedResource,
)

from ._enums import (  # type: ignore
    AttestationServiceStatus,
    CreatedByType,
    PrivateEndpointConnectionProvisioningState,
    PrivateEndpointServiceConnectionStatus,
    PublicNetworkAccessType,
    TpmAttestationAuthenticationType,
)
from ._patch import __all__ as _patch_all
from ._patch import *
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "AttestationProvider",
    "AttestationProviderListResult",
    "AttestationServiceCreationParams",
    "AttestationServiceCreationSpecificParams",
    "AttestationServicePatchParams",
    "AttestationServicePatchSpecificParams",
    "CloudError",
    "CloudErrorBody",
    "JSONWebKey",
    "JSONWebKeySet",
    "LogSpecification",
    "OperationProperties",
    "OperationsDefinition",
    "OperationsDisplayDefinition",
    "PrivateEndpoint",
    "PrivateEndpointConnection",
    "PrivateEndpointConnectionProperties",
    "PrivateLinkResource",
    "PrivateLinkResourceListResult",
    "PrivateLinkResourceProperties",
    "PrivateLinkServiceConnectionState",
    "Resource",
    "ServiceSpecification",
    "StatusResult",
    "SystemData",
    "TrackedResource",
    "AttestationServiceStatus",
    "CreatedByType",
    "PrivateEndpointConnectionProvisioningState",
    "PrivateEndpointServiceConnectionStatus",
    "PublicNetworkAccessType",
    "TpmAttestationAuthenticationType",
]
__all__.extend([p for p in _patch_all if p not in __all__])  # pyright: ignore
_patch_sdk()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from typing import Any

from azure.core.pipeline import policies

from .._version import VERSION


class MadeOptionalClientConfiguration:  # pylint: disable=too-many-instance-attributes
    """Configuration for MadeOptionalClient.

    Note that all parameters used to create this instance are saved as instance
    attributes.

    :param endpoint: Need to be set as '`http://localhost:3000 <http://localhost:3000>`_' in
     client. Required.
    :type endpoint: str
    :keyword version: Need to be set as 'v1' or 'v2' in client. Known values are "v2" and None.
     Default value is None. If not set, the operation's default API version will be used. Note that
     overriding this default value may result in unsupported behavior.
    :paramtype version: str or ~versioning.madeoptional.models.Versions
    """

    def __init__(self, endpoint: str, **kwargs: Any) -> None:
        version: str = kwargs.pop("version", "v2")

        if endpoint is None:
            raise ValueError("Parameter 'endpoint' must not be None.")

        self.endpoint = endpoint
        self.version = version
        kwargs.setdefault("sdk_moniker", "versioning-madeoptional/{}".format(VERSION))
        self.polling_interval = kwargs.get("polling_interval", 30)
        self._configure(**kwargs)

    def _configure(self, **kwargs: Any) -> None:
        self.user_agent_policy = kwargs.get("user_agent_policy") or policies.UserAgentPolicy(**kwargs)
        self.headers_policy = kwargs.get("headers_policy") or policies.HeadersPolicy(**kwargs)
        self.proxy_policy = kwargs.get("proxy_policy") or policies.ProxyPolicy(**kwargs)
        self.logging_policy = kwargs.get("logging_policy") or policies.NetworkTraceLoggingPolicy(**kwargs)
        self.http_logging_policy = kwargs.get("http_logging_policy") or policies.HttpLoggingPolicy(**kwargs)
        self.custom_hook_policy = kwargs.get("custom_hook_policy") or policies.CustomHookPolicy(**kwargs)
        self.redirect_policy = kwargs.get("redirect_policy") or policies.AsyncRedirectPolicy(**kwargs)
        self.retry_policy = kwargs.get("retry_policy") or policies.AsyncRetryPolicy(**kwargs)
        self.authentication_policy = kwargs.get("authentication_policy")



# coding=utf-8
from collections.abc import MutableMapping
from typing import Any, Callable, Optional, TypeVar

from corehttp.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from corehttp.rest import AsyncHttpResponse, HttpRequest
from corehttp.runtime import AsyncPipelineClient
from corehttp.runtime.pipeline import PipelineResponse

from ... import models as _models
from ..._operations._operations import build_custom_invalid_request, build_custom_valid_request
from ..._utils.model_base import _failsafe_deserialize
from ..._utils.utils import ClientMixinABC
from .._configuration import CustomClientConfiguration

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, dict[str, Any]], Any]]


class _CustomClientOperationsMixin(
    ClientMixinABC[AsyncPipelineClient[HttpRequest, AsyncHttpResponse], CustomClientConfiguration]
):

    async def valid(self, **kwargs: Any) -> None:
        """Check whether client is authenticated.

        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_custom_valid_request(
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = await self._client.pipeline.run(  # type: ignore
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    async def invalid(self, **kwargs: Any) -> None:
        """Check whether client is authenticated.

        :return: None
        :rtype: None
        :raises ~corehttp.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_custom_invalid_request(
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = False
        pipeline_response: PipelineResponse = await self._client.pipeline.run(  # type: ignore
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = None
            if response.status_code == 403:
                error = _failsafe_deserialize(_models.InvalidAuth, response)
            raise HttpResponseError(response=response, model=error)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from devtools_testutils import AzureRecordedTestCase, PowerShellPreparer
import functools
from specs.azure.clientgenerator.core.clientinitialization.individually import (
    IndividuallyNestedWithHeaderClient,
    IndividuallyNestedWithMixedClient,
    IndividuallyNestedWithMultipleClient,
    IndividuallyNestedWithParamAliasClient,
    IndividuallyNestedWithPathClient,
    IndividuallyNestedWithQueryClient,
)


class IndividuallyNestedWithPathClientTestBase(AzureRecordedTestCase):

    def create_client(self, endpoint):
        credential = self.get_credential(IndividuallyNestedWithPathClient)
        return self.create_client_from_credential(
            IndividuallyNestedWithPathClient,
            credential=credential,
            endpoint=endpoint,
        )


IndividuallyNestedWithPathPreparer = functools.partial(
    PowerShellPreparer,
    "individuallynestedwithpath",
    individuallynestedwithpath_endpoint="https://fake_individuallynestedwithpath_endpoint.com",
)


class IndividuallyNestedWithQueryClientTestBase(AzureRecordedTestCase):

    def create_client(self, endpoint):
        credential = self.get_credential(IndividuallyNestedWithQueryClient)
        return self.create_client_from_credential(
            IndividuallyNestedWithQueryClient,
            credential=credential,
            endpoint=endpoint,
        )


IndividuallyNestedWithQueryPreparer = functools.partial(
    PowerShellPreparer,
    "individuallynestedwithquery",
    individuallynestedwithquery_endpoint="https://fake_individuallynestedwithquery_endpoint.com",
)


class IndividuallyNestedWithHeaderClientTestBase(AzureRecordedTestCase):

    def create_client(self, endpoint):
        credential = self.get_credential(IndividuallyNestedWithHeaderClient)
        return self.create_client_from_credential(
            IndividuallyNestedWithHeaderClient,
            credential=credential,
            endpoint=endpoint,
        )


IndividuallyNestedWithHeaderPreparer = functools.partial(
    PowerShellPreparer,
    "individuallynestedwithheader",
    individuallynestedwithheader_endpoint="https://fake_individuallynestedwithheader_endpoint.com",
)


class IndividuallyNestedWithMultipleClientTestBase(AzureRecordedTestCase):

    def create_client(self, endpoint):
        credential = self.get_credential(IndividuallyNestedWithMultipleClient)
        return self.create_client_from_credential(
            IndividuallyNestedWithMultipleClient,
            credential=credential,
            endpoint=endpoint,
        )


IndividuallyNestedWithMultiplePreparer = functools.partial(
    PowerShellPreparer,
    "individuallynestedwithmultiple",
    individuallynestedwithmultiple_endpoint="https://fake_individuallynestedwithmultiple_endpoint.com",
)


class IndividuallyNestedWithMixedClientTestBase(AzureRecordedTestCase):

    def create_client(self, endpoint):
        credential = self.get_credential(IndividuallyNestedWithMixedClient)
        return self.create_client_from_credential(
            IndividuallyNestedWithMixedClient,
            credential=credential,
            endpoint=endpoint,
        )


IndividuallyNestedWithMixedPreparer = functools.partial(
    PowerShellPreparer,
    "individuallynestedwithmixed",
    individuallynestedwithmixed_endpoint="https://fake_individuallynestedwithmixed_endpoint.com",
)


class IndividuallyNestedWithParamAliasClientTestBase(AzureRecordedTestCase):

    def create_client(self, endpoint):
        credential = self.get_credential(IndividuallyNestedWithParamAliasClient)
        return self.create_client_from_credential(
            IndividuallyNestedWithParamAliasClient,
            credential=credential,
            endpoint=endpoint,
        )


IndividuallyNestedWithParamAliasPreparer = functools.partial(
    PowerShellPreparer,
    "individuallynestedwithparamalias",
    individuallynestedwithparamalias_endpoint="https://fake_individuallynestedwithparamalias_endpoint.com",
)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.applicationinsights.aio import ApplicationInsightsManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestApplicationInsightsManagementWorkbookTemplatesOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ApplicationInsightsManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_workbook_templates_list_by_resource_group(self, resource_group):
        response = self.client.workbook_templates.list_by_resource_group(
            resource_group_name=resource_group.name,
            api_version="2020-11-20",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_workbook_templates_get(self, resource_group):
        response = await self.client.workbook_templates.get(
            resource_group_name=resource_group.name,
            resource_name="str",
            api_version="2020-11-20",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_workbook_templates_delete(self, resource_group):
        response = await self.client.workbook_templates.delete(
            resource_group_name=resource_group.name,
            resource_name="str",
            api_version="2020-11-20",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_workbook_templates_create_or_update(self, resource_group):
        response = await self.client.workbook_templates.create_or_update(
            resource_group_name=resource_group.name,
            resource_name="str",
            workbook_template_properties={
                "location": "str",
                "author": "str",
                "galleries": [{"category": "str", "name": "str", "order": 0, "resourceType": "str", "type": "str"}],
                "id": "str",
                "localized": {
                    "str": [
                        {
                            "galleries": [
                                {"category": "str", "name": "str", "order": 0, "resourceType": "str", "type": "str"}
                            ],
                            "templateData": {},
                        }
                    ]
                },
                "name": "str",
                "priority": 0,
                "tags": {"str": "str"},
                "templateData": {},
                "type": "str",
            },
            api_version="2020-11-20",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_workbook_templates_update(self, resource_group):
        response = await self.client.workbook_templates.update(
            resource_group_name=resource_group.name,
            resource_name="str",
            api_version="2020-11-20",
        )

        # please add some check logic here by yourself
        # ...



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from enum import Enum
from azure.core import CaseInsensitiveEnumMeta


class AccessRecommendationType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The feature- generated recommendation shown to the reviewer."""

    APPROVE = "Approve"
    """Approve."""
    DENY = "Deny"
    """Deny."""
    NO_INFO_AVAILABLE = "NoInfoAvailable"
    """NoInfoAvailable."""


class AccessReviewActorIdentityType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The identity type : user/servicePrincipal."""

    USER = "user"
    """user."""
    SERVICE_PRINCIPAL = "servicePrincipal"
    """servicePrincipal."""


class AccessReviewApplyResult(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The outcome of applying the decision."""

    NEW = "New"
    """New."""
    APPLYING = "Applying"
    """Applying."""
    APPLIED_SUCCESSFULLY = "AppliedSuccessfully"
    """AppliedSuccessfully."""
    APPLIED_WITH_UNKNOWN_FAILURE = "AppliedWithUnknownFailure"
    """AppliedWithUnknownFailure."""
    APPLIED_SUCCESSFULLY_BUT_OBJECT_NOT_FOUND = "AppliedSuccessfullyButObjectNotFound"
    """AppliedSuccessfullyButObjectNotFound."""
    APPLY_NOT_SUPPORTED = "ApplyNotSupported"
    """ApplyNotSupported."""


class AccessReviewDecisionInsightType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of insight."""

    USER_SIGN_IN_INSIGHT = "userSignInInsight"
    """userSignInInsight."""


class AccessReviewDecisionPrincipalResourceMembershipType(  # pylint: disable=name-too-long
    str, Enum, metaclass=CaseInsensitiveEnumMeta
):
    """Type of AccessReviewDecisionPrincipalResourceMembershipType."""

    DIRECT = "direct"
    """direct."""
    INDIRECT = "indirect"
    """indirect."""


class AccessReviewHistoryDefinitionStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """This read-only field specifies the of the requested review history data. This is either
    requested, in-progress, done or error.
    """

    REQUESTED = "Requested"
    """Requested."""
    IN_PROGRESS = "InProgress"
    """InProgress."""
    DONE = "Done"
    """Done."""
    ERROR = "Error"
    """Error."""


class AccessReviewInstanceReviewersType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """This field specifies the type of reviewers for a review. Usually for a review, reviewers are
    explicitly assigned. However, in some cases, the reviewers may not be assigned and instead be
    chosen dynamically. For example managers review or self review.
    """

    ASSIGNED = "Assigned"
    """Assigned."""
    SELF = "Self"
    """Self."""
    MANAGERS = "Managers"
    """Managers."""


class AccessReviewInstanceStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """This read-only field specifies the status of an access review instance."""

    NOT_STARTED = "NotStarted"
    """NotStarted."""
    IN_PROGRESS = "InProgress"
    """InProgress."""
    COMPLETED = "Completed"
    """Completed."""
    APPLIED = "Applied"
    """Applied."""
    INITIALIZING = "Initializing"
    """Initializing."""
    APPLYING = "Applying"
    """Applying."""
    COMPLETING = "Completing"
    """Completing."""
    SCHEDULED = "Scheduled"
    """Scheduled."""
    AUTO_REVIEWING = "AutoReviewing"
    """AutoReviewing."""
    AUTO_REVIEWED = "AutoReviewed"
    """AutoReviewed."""
    STARTING = "Starting"
    """Starting."""


class AccessReviewRecurrencePatternType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The recurrence type : weekly, monthly, etc."""

    WEEKLY = "weekly"
    """weekly."""
    ABSOLUTE_MONTHLY = "absoluteMonthly"
    """absoluteMonthly."""


class AccessReviewRecurrenceRangeType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The recurrence range type. The possible values are: endDate, noEnd, numbered."""

    END_DATE = "endDate"
    """endDate."""
    NO_END = "noEnd"
    """noEnd."""
    NUMBERED = "numbered"
    """numbered."""


class AccessReviewResult(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Represents a reviewer's decision for a given review."""

    APPROVE = "Approve"
    """Approve."""
    DENY = "Deny"
    """Deny."""
    NOT_REVIEWED = "NotReviewed"
    """NotReviewed."""
    DONT_KNOW = "DontKnow"
    """DontKnow."""
    NOT_NOTIFIED = "NotNotified"
    """NotNotified."""


class AccessReviewReviewerType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The identity type : user/servicePrincipal."""

    USER = "user"
    """user."""
    SERVICE_PRINCIPAL = "servicePrincipal"
    """servicePrincipal."""


class AccessReviewScheduleDefinitionReviewersType(  # pylint: disable=name-too-long
    str, Enum, metaclass=CaseInsensitiveEnumMeta
):
    """This field specifies the type of reviewers for a review. Usually for a review, reviewers are
    explicitly assigned. However, in some cases, the reviewers may not be assigned and instead be
    chosen dynamically. For example managers review or self review.
    """

    ASSIGNED = "Assigned"
    """Assigned."""
    SELF = "Self"
    """Self."""
    MANAGERS = "Managers"
    """Managers."""


class AccessReviewScheduleDefinitionStatus(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """This read-only field specifies the status of an accessReview."""

    NOT_STARTED = "NotStarted"
    """NotStarted."""
    IN_PROGRESS = "InProgress"
    """InProgress."""
    COMPLETED = "Completed"
    """Completed."""
    APPLIED = "Applied"
    """Applied."""
    INITIALIZING = "Initializing"
    """Initializing."""
    APPLYING = "Applying"
    """Applying."""
    COMPLETING = "Completing"
    """Completing."""
    SCHEDULED = "Scheduled"
    """Scheduled."""
    AUTO_REVIEWING = "AutoReviewing"
    """AutoReviewing."""
    AUTO_REVIEWED = "AutoReviewed"
    """AutoReviewed."""
    STARTING = "Starting"
    """Starting."""


class AccessReviewScopeAssignmentState(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The role assignment state eligible/active to review."""

    ELIGIBLE = "eligible"
    """eligible."""
    ACTIVE = "active"
    """active."""


class AccessReviewScopePrincipalType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The identity type user/servicePrincipal to review."""

    USER = "user"
    """user."""
    GUEST_USER = "guestUser"
    """guestUser."""
    SERVICE_PRINCIPAL = "servicePrincipal"
    """servicePrincipal."""
    USER_GROUP = "user,group"
    """user,group."""
    REDEEMED_GUEST_USER = "redeemedGuestUser"
    """redeemedGuestUser."""


class ApprovalMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of rule."""

    SINGLE_STAGE = "SingleStage"
    """SingleStage."""
    SERIAL = "Serial"
    """Serial."""
    PARALLEL = "Parallel"
    """Parallel."""
    NO_APPROVAL = "NoApproval"
    """NoApproval."""


class AssignmentType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Assignment type of the role assignment schedule."""

    ACTIVATED = "Activated"
    """Activated."""
    ASSIGNED = "Assigned"
    """Assigned."""


class CommonUserType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of user."""

    USER = "User"
    """User."""
    GROUP = "Group"
    """Group."""


class CreatedByType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The kind of entity that created the resource."""

    USER = "User"
    """The entity was created by a user."""
    APPLICATION = "Application"
    """The entity was created by an application."""
    MANAGED_IDENTITY = "ManagedIdentity"
    """The entity was created by a managed identity."""
    KEY = "Key"
    """The entity was created by a key."""


class DecisionResourceType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of resource."""

    AZURE_ROLE = "azureRole"
    """azureRole."""


class DecisionTargetType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of decision target : User/ServicePrincipal."""

    USER = "user"
    """user."""
    SERVICE_PRINCIPAL = "servicePrincipal"
    """servicePrincipal."""


class DefaultDecisionType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """This specifies the behavior for the autoReview feature when an access review completes."""

    APPROVE = "Approve"
    """Approve."""
    DENY = "Deny"
    """Deny."""
    RECOMMENDATION = "Recommendation"
    """Recommendation."""


class DenyAssignmentEffect(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The effect of the deny assignment. 'enforced' blocks access, 'audit' logs without blocking."""

    ENFORCED = "enforced"
    """enforced."""
    AUDIT = "audit"
    """audit."""


class EnablementRules(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of enablement rule."""

    MULTI_FACTOR_AUTHENTICATION = "MultiFactorAuthentication"
    """MultiFactorAuthentication."""
    JUSTIFICATION = "Justification"
    """Justification."""
    TICKETING = "Ticketing"
    """Ticketing."""


class ExcludedPrincipalTypes(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of ExcludedPrincipalTypes."""

    SERVICE_PRINCIPALS_AS_TARGET = "ServicePrincipalsAsTarget"
    """ServicePrincipalsAsTarget."""
    SERVICE_PRINCIPALS_AS_REQUESTOR = "ServicePrincipalsAsRequestor"
    """ServicePrincipalsAsRequestor."""


class MemberType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Membership type of the role assignment schedule."""

    INHERITED = "Inherited"
    """Inherited."""
    DIRECT = "Direct"
    """Direct."""
    GROUP = "Group"
    """Group."""


class NotificationDeliveryMechanism(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of notification."""

    EMAIL = "Email"
    """Email."""


class NotificationLevel(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The notification level."""

    NONE = "None"
    """None."""
    CRITICAL = "Critical"
    """Critical."""
    ALL = "All"
    """All."""


class PIMOnlyMode(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Determines whether the setting is enabled, disabled or report only."""

    DISABLED = "Disabled"
    """Disabled."""
    ENABLED = "Enabled"
    """Enabled."""
    REPORT_ONLY = "ReportOnly"
    """ReportOnly."""


class PrincipalType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The principal type of the assigned principal ID."""

    USER = "User"
    """User."""
    GROUP = "Group"
    """Group."""
    SERVICE_PRINCIPAL = "ServicePrincipal"
    """ServicePrincipal."""
    FOREIGN_GROUP = "ForeignGroup"
    """ForeignGroup."""
    DEVICE = "Device"
    """Device."""


class RecipientType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The recipient type."""

    REQUESTOR = "Requestor"
    """Requestor."""
    APPROVER = "Approver"
    """Approver."""
    ADMIN = "Admin"
    """Admin."""


class RecordAllDecisionsResult(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The decision to make. Approvers can take action of Approve/Deny."""

    APPROVE = "Approve"
    """Approve."""
    DENY = "Deny"
    """Deny."""


class RequestType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of the role assignment schedule request. Eg: SelfActivate, AdminAssign etc."""

    ADMIN_ASSIGN = "AdminAssign"
    """AdminAssign."""
    ADMIN_REMOVE = "AdminRemove"
    """AdminRemove."""
    ADMIN_UPDATE = "AdminUpdate"
    """AdminUpdate."""
    ADMIN_EXTEND = "AdminExtend"
    """AdminExtend."""
    ADMIN_RENEW = "AdminRenew"
    """AdminRenew."""
    SELF_ACTIVATE = "SelfActivate"
    """SelfActivate."""
    SELF_DEACTIVATE = "SelfDeactivate"
    """SelfDeactivate."""
    SELF_EXTEND = "SelfExtend"
    """SelfExtend."""
    SELF_RENEW = "SelfRenew"
    """SelfRenew."""


class RoleManagementPolicyRuleType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of rule."""

    ROLE_MANAGEMENT_POLICY_APPROVAL_RULE = "RoleManagementPolicyApprovalRule"
    """RoleManagementPolicyApprovalRule."""
    ROLE_MANAGEMENT_POLICY_AUTHENTICATION_CONTEXT_RULE = "RoleManagementPolicyAuthenticationContextRule"
    """RoleManagementPolicyAuthenticationContextRule."""
    ROLE_MANAGEMENT_POLICY_ENABLEMENT_RULE = "RoleManagementPolicyEnablementRule"
    """RoleManagementPolicyEnablementRule."""
    ROLE_MANAGEMENT_POLICY_EXPIRATION_RULE = "RoleManagementPolicyExpirationRule"
    """RoleManagementPolicyExpirationRule."""
    ROLE_MANAGEMENT_POLICY_NOTIFICATION_RULE = "RoleManagementPolicyNotificationRule"
    """RoleManagementPolicyNotificationRule."""
    ROLE_MANAGEMENT_POLICY_PIM_ONLY_MODE_RULE = "RoleManagementPolicyPimOnlyModeRule"
    """RoleManagementPolicyPimOnlyModeRule."""


class SeverityLevel(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Severity level of the alert."""

    LOW = "Low"
    """Low."""
    MEDIUM = "Medium"
    """Medium."""
    HIGH = "High"
    """High."""


class Status(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The status of the role assignment schedule."""

    ACCEPTED = "Accepted"
    """Accepted."""
    PENDING_EVALUATION = "PendingEvaluation"
    """PendingEvaluation."""
    GRANTED = "Granted"
    """Granted."""
    DENIED = "Denied"
    """Denied."""
    PENDING_PROVISIONING = "PendingProvisioning"
    """PendingProvisioning."""
    PROVISIONED = "Provisioned"
    """Provisioned."""
    PENDING_REVOCATION = "PendingRevocation"
    """PendingRevocation."""
    REVOKED = "Revoked"
    """Revoked."""
    CANCELED = "Canceled"
    """Canceled."""
    FAILED = "Failed"
    """Failed."""
    PENDING_APPROVAL_PROVISIONING = "PendingApprovalProvisioning"
    """PendingApprovalProvisioning."""
    PENDING_APPROVAL = "PendingApproval"
    """PendingApproval."""
    FAILED_AS_RESOURCE_IS_LOCKED = "FailedAsResourceIsLocked"
    """FailedAsResourceIsLocked."""
    PENDING_ADMIN_DECISION = "PendingAdminDecision"
    """PendingAdminDecision."""
    ADMIN_APPROVED = "AdminApproved"
    """AdminApproved."""
    ADMIN_DENIED = "AdminDenied"
    """AdminDenied."""
    TIMED_OUT = "TimedOut"
    """TimedOut."""
    PROVISIONING_STARTED = "ProvisioningStarted"
    """ProvisioningStarted."""
    INVALID = "Invalid"
    """Invalid."""
    PENDING_SCHEDULE_CREATION = "PendingScheduleCreation"
    """PendingScheduleCreation."""
    SCHEDULE_CREATED = "ScheduleCreated"
    """ScheduleCreated."""
    PENDING_EXTERNAL_PROVISIONING = "PendingExternalProvisioning"
    """PendingExternalProvisioning."""


class Type(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """Type of the role assignment schedule expiration."""

    AFTER_DURATION = "AfterDuration"
    """AfterDuration."""
    AFTER_DATE_TIME = "AfterDateTime"
    """AfterDateTime."""
    NO_EXPIRATION = "NoExpiration"
    """NoExpiration."""


class UserType(str, Enum, metaclass=CaseInsensitiveEnumMeta):
    """The type of user."""

    USER = "User"
    """User."""
    GROUP = "Group"
    """Group."""
    SERVICE_PRINCIPAL = "ServicePrincipal"
    """ServicePrincipal."""



# pylint: disable=line-too-long,useless-suppression,too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from collections.abc import MutableMapping
from io import IOBase
from typing import Any, Callable, Dict, IO, Optional, TypeVar, Union, overload

from azure.core import PipelineClient
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.paging import ItemPaged
from azure.core.pipeline import PipelineResponse
from azure.core.rest import HttpRequest, HttpResponse
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat

from .. import models as _models
from .._configuration import ApplicationInsightsManagementClientConfiguration
from .._utils.serialization import Deserializer, Serializer

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_list_request(subscription_id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2020-02-02"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop("template_url", "/subscriptions/{subscriptionId}/providers/Microsoft.Insights/components")
    path_format_arguments = {
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_list_by_resource_group_request(resource_group_name: str, subscription_id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2020-02-02"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Insights/components",
    )
    path_format_arguments = {
        "resourceGroupName": _SERIALIZER.url(
            "resource_group_name", resource_group_name, "str", max_length=90, min_length=1
        ),
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_delete_request(
    resource_group_name: str, resource_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2020-02-02"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Insights/components/{resourceName}",
    )
    path_format_arguments = {
        "resourceGroupName": _SERIALIZER.url(
            "resource_group_name", resource_group_name, "str", max_length=90, min_length=1
        ),
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
        "resourceName": _SERIALIZER.url("resource_name", resource_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="DELETE", url=_url, params=_params, headers=_headers, **kwargs)


def build_get_request(resource_group_name: str, resource_name: str, subscription_id: str, **kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2020-02-02"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Insights/components/{resourceName}",
    )
    path_format_arguments = {
        "resourceGroupName": _SERIALIZER.url(
            "resource_group_name", resource_group_name, "str", max_length=90, min_length=1
        ),
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
        "resourceName": _SERIALIZER.url("resource_name", resource_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_create_or_update_request(
    resource_group_name: str, resource_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2020-02-02"))
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Insights/components/{resourceName}",
    )
    path_format_arguments = {
        "resourceGroupName": _SERIALIZER.url(
            "resource_group_name", resource_group_name, "str", max_length=90, min_length=1
        ),
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
        "resourceName": _SERIALIZER.url("resource_name", resource_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PUT", url=_url, params=_params, headers=_headers, **kwargs)


def build_update_tags_request(
    resource_group_name: str, resource_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2020-02-02"))
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Insights/components/{resourceName}",
    )
    path_format_arguments = {
        "resourceGroupName": _SERIALIZER.url(
            "resource_group_name", resource_group_name, "str", max_length=90, min_length=1
        ),
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
        "resourceName": _SERIALIZER.url("resource_name", resource_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="PATCH", url=_url, params=_params, headers=_headers, **kwargs)


def build_purge_request(
    resource_group_name: str, resource_name: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2020-02-02"))
    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Insights/components/{resourceName}/purge",
    )
    path_format_arguments = {
        "resourceGroupName": _SERIALIZER.url(
            "resource_group_name", resource_group_name, "str", max_length=90, min_length=1
        ),
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
        "resourceName": _SERIALIZER.url("resource_name", resource_name, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_get_purge_status_request(
    resource_group_name: str, resource_name: str, purge_id: str, subscription_id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2020-02-02"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = kwargs.pop(
        "template_url",
        "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Insights/components/{resourceName}/operations/{purgeId}",
    )
    path_format_arguments = {
        "resourceGroupName": _SERIALIZER.url(
            "resource_group_name", resource_group_name, "str", max_length=90, min_length=1
        ),
        "subscriptionId": _SERIALIZER.url("subscription_id", subscription_id, "str", min_length=1),
        "resourceName": _SERIALIZER.url("resource_name", resource_name, "str"),
        "purgeId": _SERIALIZER.url("purge_id", purge_id, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


class ComponentsOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.applicationinsights.ApplicationInsightsManagementClient`'s
        :attr:`components` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client: PipelineClient = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config: ApplicationInsightsManagementClientConfiguration = (
            input_args.pop(0) if input_args else kwargs.pop("config")
        )
        self._serialize: Serializer = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize: Deserializer = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @distributed_trace
    def list(self, **kwargs: Any) -> ItemPaged["_models.ApplicationInsightsComponent"]:
        """Gets a list of all Application Insights components within a subscription.

        :return: An iterator like instance of either ApplicationInsightsComponent or the result of
         cls(response)
        :rtype:
         ~azure.core.paging.ItemPaged[~azure.mgmt.applicationinsights.models.ApplicationInsightsComponent]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2020-02-02"))
        cls: ClsType[_models.ApplicationInsightsComponentListResult] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_list_request(
                    subscription_id=self._config.subscription_id,
                    api_version=api_version,
                    headers=_headers,
                    params=_params,
                )
                _request.url = self._client.format_url(_request.url)

            else:
                _request = HttpRequest("GET", next_link)
                _request.url = self._client.format_url(_request.url)
                _request.method = "GET"
            return _request

        def extract_data(pipeline_response):
            deserialized = self._deserialize("ApplicationInsightsComponentListResult", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, iter(list_of_elem)

        def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.ErrorResponseComponents, pipeline_response)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response

        return ItemPaged(get_next, extract_data)

    @distributed_trace
    def list_by_resource_group(
        self, resource_group_name: str, **kwargs: Any
    ) -> ItemPaged["_models.ApplicationInsightsComponent"]:
        """Gets a list of Application Insights components within a resource group.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :return: An iterator like instance of either ApplicationInsightsComponent or the result of
         cls(response)
        :rtype:
         ~azure.core.paging.ItemPaged[~azure.mgmt.applicationinsights.models.ApplicationInsightsComponent]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2020-02-02"))
        cls: ClsType[_models.ApplicationInsightsComponentListResult] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_list_by_resource_group_request(
                    resource_group_name=resource_group_name,
                    subscription_id=self._config.subscription_id,
                    api_version=api_version,
                    headers=_headers,
                    params=_params,
                )
                _request.url = self._client.format_url(_request.url)

            else:
                _request = HttpRequest("GET", next_link)
                _request.url = self._client.format_url(_request.url)
                _request.method = "GET"
            return _request

        def extract_data(pipeline_response):
            deserialized = self._deserialize("ApplicationInsightsComponentListResult", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, iter(list_of_elem)

        def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.ErrorResponseComponents, pipeline_response)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response

        return ItemPaged(get_next, extract_data)

    @distributed_trace
    def delete(  # pylint: disable=inconsistent-return-statements
        self, resource_group_name: str, resource_name: str, **kwargs: Any
    ) -> None:
        """Deletes an Application Insights component.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param resource_name: The name of the Application Insights component resource. Required.
        :type resource_name: str
        :return: None or the result of cls(response)
        :rtype: None
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2020-02-02"))
        cls: ClsType[None] = kwargs.pop("cls", None)

        _request = build_delete_request(
            resource_group_name=resource_group_name,
            resource_name=resource_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseComponents, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        if cls:
            return cls(pipeline_response, None, {})  # type: ignore

    @distributed_trace
    def get(self, resource_group_name: str, resource_name: str, **kwargs: Any) -> _models.ApplicationInsightsComponent:
        """Returns an Application Insights component.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param resource_name: The name of the Application Insights component resource. Required.
        :type resource_name: str
        :return: ApplicationInsightsComponent or the result of cls(response)
        :rtype: ~azure.mgmt.applicationinsights.models.ApplicationInsightsComponent
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2020-02-02"))
        cls: ClsType[_models.ApplicationInsightsComponent] = kwargs.pop("cls", None)

        _request = build_get_request(
            resource_group_name=resource_group_name,
            resource_name=resource_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseComponents, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("ApplicationInsightsComponent", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def create_or_update(
        self,
        resource_group_name: str,
        resource_name: str,
        insight_properties: _models.ApplicationInsightsComponent,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ApplicationInsightsComponent:
        """Creates (or updates) an Application Insights component. Note: You cannot specify a different
        value for InstrumentationKey nor AppId in the Put operation.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param resource_name: The name of the Application Insights component resource. Required.
        :type resource_name: str
        :param insight_properties: Properties that need to be specified to create an Application
         Insights component. Required.
        :type insight_properties: ~azure.mgmt.applicationinsights.models.ApplicationInsightsComponent
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: ApplicationInsightsComponent or the result of cls(response)
        :rtype: ~azure.mgmt.applicationinsights.models.ApplicationInsightsComponent
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def create_or_update(
        self,
        resource_group_name: str,
        resource_name: str,
        insight_properties: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ApplicationInsightsComponent:
        """Creates (or updates) an Application Insights component. Note: You cannot specify a different
        value for InstrumentationKey nor AppId in the Put operation.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param resource_name: The name of the Application Insights component resource. Required.
        :type resource_name: str
        :param insight_properties: Properties that need to be specified to create an Application
         Insights component. Required.
        :type insight_properties: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: ApplicationInsightsComponent or the result of cls(response)
        :rtype: ~azure.mgmt.applicationinsights.models.ApplicationInsightsComponent
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def create_or_update(
        self,
        resource_group_name: str,
        resource_name: str,
        insight_properties: Union[_models.ApplicationInsightsComponent, IO[bytes]],
        **kwargs: Any
    ) -> _models.ApplicationInsightsComponent:
        """Creates (or updates) an Application Insights component. Note: You cannot specify a different
        value for InstrumentationKey nor AppId in the Put operation.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param resource_name: The name of the Application Insights component resource. Required.
        :type resource_name: str
        :param insight_properties: Properties that need to be specified to create an Application
         Insights component. Is either a ApplicationInsightsComponent type or a IO[bytes] type.
         Required.
        :type insight_properties: ~azure.mgmt.applicationinsights.models.ApplicationInsightsComponent
         or IO[bytes]
        :return: ApplicationInsightsComponent or the result of cls(response)
        :rtype: ~azure.mgmt.applicationinsights.models.ApplicationInsightsComponent
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2020-02-02"))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.ApplicationInsightsComponent] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(insight_properties, (IOBase, bytes)):
            _content = insight_properties
        else:
            _json = self._serialize.body(insight_properties, "ApplicationInsightsComponent")

        _request = build_create_or_update_request(
            resource_group_name=resource_group_name,
            resource_name=resource_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseComponents, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("ApplicationInsightsComponent", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def update_tags(
        self,
        resource_group_name: str,
        resource_name: str,
        component_tags: _models.TagsResource,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ApplicationInsightsComponent:
        """Updates an existing component's tags. To update other fields use the CreateOrUpdate method.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param resource_name: The name of the Application Insights component resource. Required.
        :type resource_name: str
        :param component_tags: Updated tag information to set into the component instance. Required.
        :type component_tags: ~azure.mgmt.applicationinsights.models.TagsResource
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: ApplicationInsightsComponent or the result of cls(response)
        :rtype: ~azure.mgmt.applicationinsights.models.ApplicationInsightsComponent
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def update_tags(
        self,
        resource_group_name: str,
        resource_name: str,
        component_tags: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ApplicationInsightsComponent:
        """Updates an existing component's tags. To update other fields use the CreateOrUpdate method.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param resource_name: The name of the Application Insights component resource. Required.
        :type resource_name: str
        :param component_tags: Updated tag information to set into the component instance. Required.
        :type component_tags: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: ApplicationInsightsComponent or the result of cls(response)
        :rtype: ~azure.mgmt.applicationinsights.models.ApplicationInsightsComponent
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def update_tags(
        self,
        resource_group_name: str,
        resource_name: str,
        component_tags: Union[_models.TagsResource, IO[bytes]],
        **kwargs: Any
    ) -> _models.ApplicationInsightsComponent:
        """Updates an existing component's tags. To update other fields use the CreateOrUpdate method.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param resource_name: The name of the Application Insights component resource. Required.
        :type resource_name: str
        :param component_tags: Updated tag information to set into the component instance. Is either a
         TagsResource type or a IO[bytes] type. Required.
        :type component_tags: ~azure.mgmt.applicationinsights.models.TagsResource or IO[bytes]
        :return: ApplicationInsightsComponent or the result of cls(response)
        :rtype: ~azure.mgmt.applicationinsights.models.ApplicationInsightsComponent
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2020-02-02"))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.ApplicationInsightsComponent] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(component_tags, (IOBase, bytes)):
            _content = component_tags
        else:
            _json = self._serialize.body(component_tags, "TagsResource")

        _request = build_update_tags_request(
            resource_group_name=resource_group_name,
            resource_name=resource_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseComponents, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("ApplicationInsightsComponent", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @overload
    def purge(
        self,
        resource_group_name: str,
        resource_name: str,
        body: _models.ComponentPurgeBody,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ComponentPurgeResponse:
        """Purges data in an Application Insights component by a set of user-defined filters.

        In order to manage system resources, purge requests are throttled at 50 requests per hour. You
        should batch the execution of purge requests by sending a single command whose predicate
        includes all user identities that require purging. Use the in operator to specify multiple
        identities. You should run the query prior to using for a purge request to verify that the
        results are expected.
        Note: this operation is intended for Classic resources, for  workspace-based Application
        Insights resource please run purge operation (directly on the
        workspace)(https://docs.microsoft.com/en-us/rest/api/loganalytics/workspace-purge/purge) ,
        scoped to specific resource id.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param resource_name: The name of the Application Insights component resource. Required.
        :type resource_name: str
        :param body: Describes the body of a request to purge data in a single table of an Application
         Insights component. Required.
        :type body: ~azure.mgmt.applicationinsights.models.ComponentPurgeBody
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: ComponentPurgeResponse or the result of cls(response)
        :rtype: ~azure.mgmt.applicationinsights.models.ComponentPurgeResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def purge(
        self,
        resource_group_name: str,
        resource_name: str,
        body: IO[bytes],
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.ComponentPurgeResponse:
        """Purges data in an Application Insights component by a set of user-defined filters.

        In order to manage system resources, purge requests are throttled at 50 requests per hour. You
        should batch the execution of purge requests by sending a single command whose predicate
        includes all user identities that require purging. Use the in operator to specify multiple
        identities. You should run the query prior to using for a purge request to verify that the
        results are expected.
        Note: this operation is intended for Classic resources, for  workspace-based Application
        Insights resource please run purge operation (directly on the
        workspace)(https://docs.microsoft.com/en-us/rest/api/loganalytics/workspace-purge/purge) ,
        scoped to specific resource id.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param resource_name: The name of the Application Insights component resource. Required.
        :type resource_name: str
        :param body: Describes the body of a request to purge data in a single table of an Application
         Insights component. Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: ComponentPurgeResponse or the result of cls(response)
        :rtype: ~azure.mgmt.applicationinsights.models.ComponentPurgeResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def purge(
        self,
        resource_group_name: str,
        resource_name: str,
        body: Union[_models.ComponentPurgeBody, IO[bytes]],
        **kwargs: Any
    ) -> _models.ComponentPurgeResponse:
        """Purges data in an Application Insights component by a set of user-defined filters.

        In order to manage system resources, purge requests are throttled at 50 requests per hour. You
        should batch the execution of purge requests by sending a single command whose predicate
        includes all user identities that require purging. Use the in operator to specify multiple
        identities. You should run the query prior to using for a purge request to verify that the
        results are expected.
        Note: this operation is intended for Classic resources, for  workspace-based Application
        Insights resource please run purge operation (directly on the
        workspace)(https://docs.microsoft.com/en-us/rest/api/loganalytics/workspace-purge/purge) ,
        scoped to specific resource id.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param resource_name: The name of the Application Insights component resource. Required.
        :type resource_name: str
        :param body: Describes the body of a request to purge data in a single table of an Application
         Insights component. Is either a ComponentPurgeBody type or a IO[bytes] type. Required.
        :type body: ~azure.mgmt.applicationinsights.models.ComponentPurgeBody or IO[bytes]
        :return: ComponentPurgeResponse or the result of cls(response)
        :rtype: ~azure.mgmt.applicationinsights.models.ComponentPurgeResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2020-02-02"))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.ComponentPurgeResponse] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _json = self._serialize.body(body, "ComponentPurgeBody")

        _request = build_purge_request(
            resource_group_name=resource_group_name,
            resource_name=resource_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [202]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseComponents, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("ComponentPurgeResponse", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def get_purge_status(
        self, resource_group_name: str, resource_name: str, purge_id: str, **kwargs: Any
    ) -> _models.ComponentPurgeStatusResponse:
        """Get status for an ongoing purge operation.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param resource_name: The name of the Application Insights component resource. Required.
        :type resource_name: str
        :param purge_id: In a purge status request, this is the Id of the operation the status of which
         is returned. Required.
        :type purge_id: str
        :return: ComponentPurgeStatusResponse or the result of cls(response)
        :rtype: ~azure.mgmt.applicationinsights.models.ComponentPurgeStatusResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2020-02-02"))
        cls: ClsType[_models.ComponentPurgeStatusResponse] = kwargs.pop("cls", None)

        _request = build_get_purge_status_request(
            resource_group_name=resource_group_name,
            resource_name=resource_name,
            purge_id=purge_id,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            headers=_headers,
            params=_params,
        )
        _request.url = self._client.format_url(_request.url)

        _stream = False
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponseComponents, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("ComponentPurgeStatusResponse", pipeline_response.http_response)

        if cls:
            return cls(pipeline_response, deserialized, {})  # type: ignore

        return deserialized  # type: ignore



# pylint: disable=too-many-lines,line-too-long,useless-suppression
# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
"""
Tests for ResponsesInstrumentor with Code Interpreter tool (async).
"""

import os
from io import BytesIO
import pytest
from gen_ai_trace_verifier import GenAiTraceVerifier  # pylint: disable=import-error
from devtools_testutils.aio import recorded_by_proxy_async
from devtools_testutils import RecordedTransport
from test_base import servicePreparer
from test_ai_instrumentor_base import (  # pylint: disable=import-error
    TestAiAgentsInstrumentorBase,
    CONTENT_TRACING_ENV_VARIABLE,
)
from azure.ai.projects.telemetry import AIProjectInstrumentor, _utils
from azure.ai.projects.telemetry._utils import (
    OPERATION_NAME_INVOKE_AGENT,
    SPAN_NAME_INVOKE_AGENT,
    _set_use_message_events,
    RESPONSES_PROVIDER,
)
from azure.ai.projects.models import (
    PromptAgentDefinition,
    CodeInterpreterTool,
    AutoCodeInterpreterToolParam,
)
from azure.core.settings import settings

settings.tracing_implementation = "OpenTelemetry"
_utils._span_impl_type = settings.tracing_implementation()  # pylint: disable=not-callable


class TestResponsesInstrumentorCodeInterpreterAsync(TestAiAgentsInstrumentorBase):
    """
    Test suite for Code Interpreter agent telemetry instrumentation (async).

    This class tests OpenTelemetry trace generation when using Code Interpreter tool
    with both content recording enabled and disabled, in both streaming and non-streaming modes.
    """

    # pylint: disable=too-many-nested-blocks

    # ========================================
    # Async Code Interpreter Agent Tests - Non-Streaming
    # ========================================

    @pytest.mark.usefixtures("instrument_with_content")
    @servicePreparer()
    @recorded_by_proxy_async(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    async def test_async_code_interpreter_non_streaming_with_content_recording(
        self, **kwargs
    ):  # pylint: disable=too-many-locals,too-many-branches,too-many-statements,too-many-nested-blocks
        """Test asynchronous Code Interpreter agent with content recording enabled."""
        self.cleanup()
        _set_use_message_events(True)
        os.environ.update(
            {
                CONTENT_TRACING_ENV_VARIABLE: "True",
                "AZURE_TRACING_GEN_AI_INSTRUMENT_RESPONSES_API": "True",
            }
        )
        self.setup_telemetry()
        assert AIProjectInstrumentor().is_content_recording_enabled()
        assert AIProjectInstrumentor().is_instrumented()

        project_client = self.create_async_client(operation_group="tracing", **kwargs)
        deployment_name = kwargs.get("foundry_model_name")
        assert deployment_name is not None

        async with project_client:
            openai_client = project_client.get_openai_client()

            # Create CSV data document
            csv_data = """sector,name,operating_profit
TRANSPORTATION,Contoso shipping,850000
TRANSPORTATION,Contoso rail,920000
TRANSPORTATION,Contoso air,1100000
"""

            csv_file = BytesIO(csv_data.encode("utf-8"))
            csv_file.name = "transportation_data.csv"

            # Upload file for code interpreter
            file = await openai_client.files.create(purpose="assistants", file=csv_file)
            assert file.id is not None

            # Create agent with Code Interpreter tool
            agent = await project_client.agents.create_version(
                agent_name="MyAgent",
                definition=PromptAgentDefinition(
                    model=deployment_name,
                    instructions="You are a helpful assistant that can execute Python code to analyze data.",
                    tools=[CodeInterpreterTool(container=AutoCodeInterpreterToolParam(file_ids=[file.id]))],
                ),
            )

            try:
                conversation = await openai_client.conversations.create()

                # Ask question that triggers code interpreter
                _ = await openai_client.responses.create(
                    conversation=conversation.id,
                    input="Calculate the average operating profit from the transportation data",
                    extra_body={"agent_reference": {"name": agent.name, "type": "agent_reference"}},
                )

                # Explicitly call and iterate through conversation items
                items = await openai_client.conversations.items.list(conversation_id=conversation.id)
                async for _ in items:
                    pass

                # Check spans
                self.exporter.force_flush()
                spans = self.exporter.get_spans_by_name(f"{SPAN_NAME_INVOKE_AGENT} {agent.name}")
                assert len(spans) == 1, "Should have one response span"

                # Validate response span
                span = spans[0]
                assert span.attributes is not None
                response_id = span.attributes.get("gen_ai.response.id")
                assert response_id is not None

                expected_attributes = [
                    ("az.namespace", "Microsoft.CognitiveServices"),
                    ("gen_ai.operation.name", OPERATION_NAME_INVOKE_AGENT),
                    ("gen_ai.provider.name", RESPONSES_PROVIDER),
                    ("server.address", ""),
                    ("gen_ai.conversation.id", conversation.id),
                    ("gen_ai.agent.name", agent.name),
                    ("gen_ai.response.model", deployment_name),
                    ("gen_ai.response.id", response_id),
                    ("gen_ai.usage.input_tokens", "+"),
                    ("gen_ai.usage.output_tokens", "+"),
                ]
                assert GenAiTraceVerifier().check_span_attributes(span, expected_attributes)

                # Comprehensive event validation - verify content IS present
                from collections.abc import Mapping
                import json

                found_code_interpreter_call = False
                found_text_response = False

                for event in span.events:
                    if event.name == "gen_ai.input.messages":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)
                        for entry in data:
                            if entry.get("role") == "user":
                                parts = entry.get("parts")
                                for part in parts:
                                    if part.get("type") == "text":
                                        assert "content" in part and isinstance(
                                            part["content"], str
                                        ), "Text content should be present when content recording is enabled"

                    elif event.name == "gen_ai.output.messages":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)

                        for entry in data:
                            parts = entry.get("parts")
                            if parts:
                                for part in parts:
                                    if part.get("type") == "tool_call":
                                        tool_content = part.get("content")
                                        if tool_content and tool_content.get("type") == "code_interpreter_call":
                                            found_code_interpreter_call = True
                                            assert "id" in tool_content, "code_interpreter_call should have id"
                                            # With content recording, code should be present
                                            assert (
                                                "code" in tool_content
                                            ), "code should be present when content recording is enabled"
                                    elif part.get("type") == "text":
                                        found_text_response = True
                                        assert (
                                            "content" in part
                                        ), "text content should be present when content recording is enabled"

                assert found_code_interpreter_call, "Should have found code_interpreter_call in output"
                assert found_text_response, "Should have found text response in output"

                # Check list_conversation_items span
                list_spans = self.exporter.get_spans_by_name("list_conversation_items")
                assert len(list_spans) == 1, "Should have one list_conversation_items span"
                list_span = list_spans[0]

                found_code_interpreter_in_items = False
                for event in list_span.events:
                    if event.name == "gen_ai.conversation.item":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)

                        for entry in data:
                            parts = entry.get("parts")
                            if parts:
                                for part in parts:
                                    if part.get("type") == "text":
                                        assert "content" in part, "text content should be present in conversation items"
                                    elif part.get("type") == "tool_call":
                                        tool_content = part.get("content")
                                        if tool_content and tool_content.get("type") == "code_interpreter_call":
                                            found_code_interpreter_in_items = True
                                            assert (
                                                "id" in tool_content
                                            ), "code_interpreter_call should have id in conversation items"
                                            code_interpreter = tool_content.get("code_interpreter")
                                            if code_interpreter:
                                                assert (
                                                    "code" in code_interpreter
                                                ), "code should be present when content recording is enabled"
                                                assert (
                                                    "status" in code_interpreter
                                                ), "status should be present in code_interpreter"
                    else:
                        assert False, f"Unexpected event name in list_conversation_items span: {event.name}"

                assert found_code_interpreter_in_items, "Should have found code_interpreter_call in conversation items"

                # Cleanup
                await openai_client.conversations.delete(conversation_id=conversation.id)
                await openai_client.files.delete(file.id)

            finally:
                await project_client.agents.delete_version(agent_name=agent.name, agent_version=agent.version)

    @pytest.mark.usefixtures("instrument_without_content")
    @servicePreparer()
    @recorded_by_proxy_async(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    async def test_async_code_interpreter_non_streaming_without_content_recording(
        self, **kwargs
    ):  # pylint: disable=too-many-locals,too-many-branches,too-many-statements,too-many-nested-blocks
        """Test asynchronous Code Interpreter agent with content recording disabled."""
        self.cleanup()
        _set_use_message_events(True)
        os.environ.update(
            {
                CONTENT_TRACING_ENV_VARIABLE: "False",
                "AZURE_TRACING_GEN_AI_INSTRUMENT_RESPONSES_API": "True",
            }
        )
        self.setup_telemetry()
        assert not AIProjectInstrumentor().is_content_recording_enabled()
        assert AIProjectInstrumentor().is_instrumented()

        project_client = self.create_async_client(operation_group="tracing", **kwargs)
        deployment_name = kwargs.get("foundry_model_name")
        assert deployment_name is not None

        async with project_client:
            openai_client = project_client.get_openai_client()

            # Create CSV data document
            csv_data = """sector,name,operating_profit
TRANSPORTATION,Contoso shipping,850000
TRANSPORTATION,Contoso rail,920000
TRANSPORTATION,Contoso air,1100000
"""

            csv_file = BytesIO(csv_data.encode("utf-8"))
            csv_file.name = "transportation_data.csv"

            # Upload file for code interpreter
            file = await openai_client.files.create(purpose="assistants", file=csv_file)
            assert file.id is not None

            # Create agent with Code Interpreter tool
            agent = await project_client.agents.create_version(
                agent_name="MyAgent",
                definition=PromptAgentDefinition(
                    model=deployment_name,
                    instructions="You are a helpful assistant that can execute Python code to analyze data.",
                    tools=[CodeInterpreterTool(container=AutoCodeInterpreterToolParam(file_ids=[file.id]))],
                ),
            )

            try:
                conversation = await openai_client.conversations.create()

                # Ask question that triggers code interpreter
                _ = await openai_client.responses.create(
                    conversation=conversation.id,
                    input="Calculate the average operating profit from the transportation data",
                    extra_body={"agent_reference": {"name": agent.name, "type": "agent_reference"}},
                )

                # Explicitly call and iterate through conversation items
                items = await openai_client.conversations.items.list(conversation_id=conversation.id)
                async for _ in items:
                    pass

                # Check spans
                self.exporter.force_flush()
                spans = self.exporter.get_spans_by_name(f"{SPAN_NAME_INVOKE_AGENT} {agent.name}")
                assert len(spans) == 1, "Should have one response span"

                # Validate response span
                span = spans[0]
                assert span.attributes is not None
                response_id = span.attributes.get("gen_ai.response.id")
                assert response_id is not None

                expected_attributes = [
                    ("az.namespace", "Microsoft.CognitiveServices"),
                    ("gen_ai.operation.name", OPERATION_NAME_INVOKE_AGENT),
                    ("gen_ai.provider.name", RESPONSES_PROVIDER),
                    ("server.address", ""),
                    ("gen_ai.conversation.id", conversation.id),
                    ("gen_ai.agent.name", agent.name),
                    ("gen_ai.response.model", deployment_name),
                    ("gen_ai.response.id", response_id),
                    ("gen_ai.usage.input_tokens", "+"),
                    ("gen_ai.usage.output_tokens", "+"),
                ]
                assert GenAiTraceVerifier().check_span_attributes(span, expected_attributes)

                # Comprehensive event validation - verify content is NOT present
                from collections.abc import Mapping
                import json

                found_code_interpreter_call = False
                found_text_response = False

                for event in span.events:
                    if event.name == "gen_ai.input.messages":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)
                        for entry in data:
                            if entry.get("role") == "user":
                                parts = entry.get("parts")
                                for part in parts:
                                    if part.get("type") == "text":
                                        assert (
                                            "content" not in part
                                        ), "Text content should NOT be present when content recording is disabled"

                    elif event.name == "gen_ai.output.messages":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)

                        for entry in data:
                            parts = entry.get("parts")
                            if parts:
                                for part in parts:
                                    if part.get("type") == "tool_call":
                                        tool_content = part.get("content")
                                        if tool_content and tool_content.get("type") == "code_interpreter_call":
                                            found_code_interpreter_call = True
                                            assert "id" in tool_content, "code_interpreter_call should have id"
                                            # Without content recording, code should NOT be present
                                            assert (
                                                "code" not in tool_content
                                            ), "code should NOT be present when content recording is disabled"
                                    elif part.get("type") == "text":
                                        found_text_response = True
                                        assert (
                                            "content" not in part
                                        ), "text content should NOT be present when content recording is disabled"

                assert found_code_interpreter_call, "Should have found code_interpreter_call in output"
                assert found_text_response, "Should have found text response type in output"

                # Check list_conversation_items span
                list_spans = self.exporter.get_spans_by_name("list_conversation_items")
                assert len(list_spans) == 1, "Should have one list_conversation_items span"
                list_span = list_spans[0]

                found_code_interpreter_in_items = False
                for event in list_span.events:
                    if event.name == "gen_ai.conversation.item":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)

                        for entry in data:
                            parts = entry.get("parts")
                            if parts:
                                for part in parts:
                                    if part.get("type") == "text":
                                        assert (
                                            "content" not in part
                                        ), "text content should NOT be present in conversation items"
                                    elif part.get("type") == "tool_call":
                                        tool_content = part.get("content")
                                        if tool_content and tool_content.get("type") == "code_interpreter_call":
                                            found_code_interpreter_in_items = True
                                            assert (
                                                "id" in tool_content
                                            ), "code_interpreter_call should have id in conversation items"
                                            # Without content recording, code should NOT be present
                                            code_interpreter = tool_content.get("code_interpreter")
                                            if code_interpreter:
                                                assert (
                                                    "code" not in code_interpreter
                                                ), "code should NOT be present when content recording is disabled"
                    else:
                        assert False, f"Unexpected event name in list_conversation_items span: {event.name}"

                assert found_code_interpreter_in_items, "Should have found code_interpreter_call in conversation items"

                # Cleanup
                await openai_client.conversations.delete(conversation_id=conversation.id)
                await openai_client.files.delete(file.id)

            finally:
                await project_client.agents.delete_version(agent_name=agent.name, agent_version=agent.version)

    # ========================================
    # Async Code Interpreter Agent Tests - Streaming
    # ========================================

    @pytest.mark.usefixtures("instrument_with_content")
    @servicePreparer()
    @recorded_by_proxy_async(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    async def test_async_code_interpreter_streaming_with_content_recording(
        self, **kwargs
    ):  # pylint: disable=too-many-locals,too-many-branches,too-many-statements,too-many-nested-blocks
        """Test asynchronous Code Interpreter agent with streaming and content recording enabled."""
        self.cleanup()
        _set_use_message_events(True)
        os.environ.update(
            {
                CONTENT_TRACING_ENV_VARIABLE: "True",
                "AZURE_TRACING_GEN_AI_INSTRUMENT_RESPONSES_API": "True",
            }
        )
        self.setup_telemetry()
        assert AIProjectInstrumentor().is_content_recording_enabled()
        assert AIProjectInstrumentor().is_instrumented()

        project_client = self.create_async_client(operation_group="tracing", **kwargs)
        deployment_name = kwargs.get("foundry_model_name")
        assert deployment_name is not None

        async with project_client:
            openai_client = project_client.get_openai_client()

            # Create CSV data document
            csv_data = """sector,name,operating_profit
TRANSPORTATION,Contoso shipping,850000
TRANSPORTATION,Contoso rail,920000
TRANSPORTATION,Contoso air,1100000
"""

            csv_file = BytesIO(csv_data.encode("utf-8"))
            csv_file.name = "transportation_data.csv"

            # Upload file for code interpreter
            file = await openai_client.files.create(purpose="assistants", file=csv_file)
            assert file.id is not None

            # Create agent with Code Interpreter tool
            agent = await project_client.agents.create_version(
                agent_name="MyAgent",
                definition=PromptAgentDefinition(
                    model=deployment_name,
                    instructions="You are a helpful assistant that can execute Python code to analyze data.",
                    tools=[CodeInterpreterTool(container=AutoCodeInterpreterToolParam(file_ids=[file.id]))],
                ),
            )

            try:
                conversation = await openai_client.conversations.create()

                # Ask question that triggers code interpreter with streaming
                stream = await openai_client.responses.create(
                    conversation=conversation.id,
                    input="Calculate the average operating profit from the transportation data",
                    stream=True,
                    extra_body={"agent_reference": {"name": agent.name, "type": "agent_reference"}},
                )

                # Consume the stream
                async for event in stream:
                    pass

                # Explicitly call and iterate through conversation items
                items = await openai_client.conversations.items.list(conversation_id=conversation.id)
                async for _ in items:
                    pass

                # Check spans
                self.exporter.force_flush()
                spans = self.exporter.get_spans_by_name(f"{SPAN_NAME_INVOKE_AGENT} {agent.name}")
                assert len(spans) == 1, "Should have one response span"

                # Validate response span
                span = spans[0]
                assert span.attributes is not None
                response_id = span.attributes.get("gen_ai.response.id")
                assert response_id is not None

                expected_attributes = [
                    ("az.namespace", "Microsoft.CognitiveServices"),
                    ("gen_ai.operation.name", OPERATION_NAME_INVOKE_AGENT),
                    ("gen_ai.provider.name", RESPONSES_PROVIDER),
                    ("server.address", ""),
                    ("gen_ai.conversation.id", conversation.id),
                    ("gen_ai.agent.name", agent.name),
                    ("gen_ai.response.model", deployment_name),
                    ("gen_ai.response.id", response_id),
                    ("gen_ai.usage.input_tokens", "+"),
                    ("gen_ai.usage.output_tokens", "+"),
                ]
                assert GenAiTraceVerifier().check_span_attributes(span, expected_attributes)

                # Comprehensive event validation - verify content IS present
                from collections.abc import Mapping
                import json

                found_code_interpreter_call = False
                found_text_response = False

                for event in span.events:
                    if event.name == "gen_ai.input.messages":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)
                        for entry in data:
                            if entry.get("role") == "user":
                                parts = entry.get("parts")
                                for part in parts:
                                    if part.get("type") == "text":
                                        assert "content" in part and isinstance(
                                            part["content"], str
                                        ), "Text content should be present when content recording is enabled"

                    elif event.name == "gen_ai.output.messages":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)

                        for entry in data:
                            parts = entry.get("parts")
                            if parts:
                                for part in parts:
                                    if part.get("type") == "tool_call":
                                        tool_content = part.get("content")
                                        if tool_content and tool_content.get("type") == "code_interpreter_call":
                                            found_code_interpreter_call = True
                                            assert "id" in tool_content, "code_interpreter_call should have id"
                                            assert (
                                                "code" in tool_content
                                            ), "code should be present when content recording is enabled"
                                    elif part.get("type") == "text":
                                        found_text_response = True
                                        assert (
                                            "content" in part
                                        ), "text content should be present when content recording is enabled"

                assert found_code_interpreter_call, "Should have found code_interpreter_call in output"
                assert found_text_response, "Should have found text response in output"

                # Check list_conversation_items span
                list_spans = self.exporter.get_spans_by_name("list_conversation_items")
                assert len(list_spans) == 1, "Should have one list_conversation_items span"
                list_span = list_spans[0]

                found_code_interpreter_in_items = False
                for event in list_span.events:
                    if event.name == "gen_ai.conversation.item":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)

                        for entry in data:
                            parts = entry.get("parts")
                            if parts:
                                for part in parts:
                                    if part.get("type") == "text":
                                        assert "content" in part, "text content should be present in conversation items"
                                    elif part.get("type") == "tool_call":
                                        tool_content = part.get("content")
                                        if tool_content and tool_content.get("type") == "code_interpreter_call":
                                            found_code_interpreter_in_items = True
                                            assert (
                                                "id" in tool_content
                                            ), "code_interpreter_call should have id in conversation items"
                                            code_interpreter = tool_content.get("code_interpreter")
                                            if code_interpreter:
                                                assert (
                                                    "code" in code_interpreter
                                                ), "code should be present when content recording is enabled"
                                                assert (
                                                    "status" in code_interpreter
                                                ), "status should be present in code_interpreter"
                    else:
                        assert False, f"Unexpected event name in list_conversation_items span: {event.name}"

                assert found_code_interpreter_in_items, "Should have found code_interpreter_call in conversation items"

                # Cleanup
                await openai_client.conversations.delete(conversation_id=conversation.id)
                await openai_client.files.delete(file.id)

            finally:
                await project_client.agents.delete_version(agent_name=agent.name, agent_version=agent.version)

    @pytest.mark.usefixtures("instrument_without_content")
    @servicePreparer()
    @recorded_by_proxy_async(RecordedTransport.AZURE_CORE, RecordedTransport.HTTPX)
    async def test_async_code_interpreter_streaming_without_content_recording(
        self, **kwargs
    ):  # pylint: disable=too-many-locals,too-many-branches,too-many-statements,too-many-nested-blocks
        """Test asynchronous Code Interpreter agent with streaming and content recording disabled."""
        self.cleanup()
        _set_use_message_events(True)
        os.environ.update(
            {
                CONTENT_TRACING_ENV_VARIABLE: "False",
                "AZURE_TRACING_GEN_AI_INSTRUMENT_RESPONSES_API": "True",
            }
        )
        self.setup_telemetry()
        assert not AIProjectInstrumentor().is_content_recording_enabled()
        assert AIProjectInstrumentor().is_instrumented()

        project_client = self.create_async_client(operation_group="tracing", **kwargs)
        deployment_name = kwargs.get("foundry_model_name")
        assert deployment_name is not None

        async with project_client:
            openai_client = project_client.get_openai_client()

            # Create CSV data document
            csv_data = """sector,name,operating_profit
TRANSPORTATION,Contoso shipping,850000
TRANSPORTATION,Contoso rail,920000
TRANSPORTATION,Contoso air,1100000
"""

            csv_file = BytesIO(csv_data.encode("utf-8"))
            csv_file.name = "transportation_data.csv"

            # Upload file for code interpreter
            file = await openai_client.files.create(purpose="assistants", file=csv_file)
            assert file.id is not None

            # Create agent with Code Interpreter tool
            agent = await project_client.agents.create_version(
                agent_name="MyAgent",
                definition=PromptAgentDefinition(
                    model=deployment_name,
                    instructions="You are a helpful assistant that can execute Python code to analyze data.",
                    tools=[CodeInterpreterTool(container=AutoCodeInterpreterToolParam(file_ids=[file.id]))],
                ),
            )

            try:
                conversation = await openai_client.conversations.create()

                # Ask question that triggers code interpreter with streaming
                stream = await openai_client.responses.create(
                    conversation=conversation.id,
                    input="Calculate the average operating profit from the transportation data",
                    stream=True,
                    extra_body={"agent_reference": {"name": agent.name, "type": "agent_reference"}},
                )

                # Consume the stream
                async for event in stream:
                    pass

                # Explicitly call and iterate through conversation items
                items = await openai_client.conversations.items.list(conversation_id=conversation.id)
                async for _ in items:
                    pass

                # Check spans
                self.exporter.force_flush()
                spans = self.exporter.get_spans_by_name(f"{SPAN_NAME_INVOKE_AGENT} {agent.name}")
                assert len(spans) == 1, "Should have one response span"

                # Validate response span
                span = spans[0]
                assert span.attributes is not None
                response_id = span.attributes.get("gen_ai.response.id")
                assert response_id is not None

                expected_attributes = [
                    ("az.namespace", "Microsoft.CognitiveServices"),
                    ("gen_ai.operation.name", OPERATION_NAME_INVOKE_AGENT),
                    ("gen_ai.provider.name", RESPONSES_PROVIDER),
                    ("server.address", ""),
                    ("gen_ai.conversation.id", conversation.id),
                    ("gen_ai.agent.name", agent.name),
                    ("gen_ai.response.model", deployment_name),
                    ("gen_ai.response.id", response_id),
                    ("gen_ai.usage.input_tokens", "+"),
                    ("gen_ai.usage.output_tokens", "+"),
                ]
                assert GenAiTraceVerifier().check_span_attributes(span, expected_attributes)

                # Comprehensive event validation - verify content is NOT present
                from collections.abc import Mapping
                import json

                found_code_interpreter_call = False
                found_text_response = False

                for event in span.events:
                    if event.name == "gen_ai.input.messages":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)
                        for entry in data:
                            if entry.get("role") == "user":
                                parts = entry.get("parts")
                                for part in parts:
                                    if part.get("type") == "text":
                                        assert (
                                            "content" not in part
                                        ), "Text content should NOT be present when content recording is disabled"

                    elif event.name == "gen_ai.output.messages":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)

                        for entry in data:
                            parts = entry.get("parts")
                            if parts:
                                for part in parts:
                                    if part.get("type") == "tool_call":
                                        tool_content = part.get("content")
                                        if tool_content and tool_content.get("type") == "code_interpreter_call":
                                            found_code_interpreter_call = True
                                            assert "id" in tool_content, "code_interpreter_call should have id"
                                            assert (
                                                "code" not in tool_content
                                            ), "code should NOT be present when content recording is disabled"
                                    elif part.get("type") == "text":
                                        found_text_response = True
                                        assert (
                                            "content" not in part
                                        ), "text content should NOT be present when content recording is disabled"

                assert found_code_interpreter_call, "Should have found code_interpreter_call in output"
                assert found_text_response, "Should have found text response type in output"

                # Check list_conversation_items span
                list_spans = self.exporter.get_spans_by_name("list_conversation_items")
                assert len(list_spans) == 1, "Should have one list_conversation_items span"
                list_span = list_spans[0]

                found_code_interpreter_in_items = False
                for event in list_span.events:
                    if event.name == "gen_ai.conversation.item":
                        attrs = event.attributes
                        assert attrs is not None and isinstance(attrs, Mapping)
                        content = attrs.get("gen_ai.event.content")
                        assert isinstance(content, str) and content.strip() != ""
                        data = json.loads(content)

                        for entry in data:
                            parts = entry.get("parts")
                            if parts:
                                for part in parts:
                                    if part.get("type") == "text":
                                        assert (
                                            "content" not in part
                                        ), "text content should NOT be present in conversation items"
                                    elif part.get("type") == "tool_call":
                                        tool_content = part.get("content")
                                        if tool_content and tool_content.get("type") == "code_interpreter_call":
                                            found_code_interpreter_in_items = True
                                            assert (
                                                "id" in tool_content
                                            ), "code_interpreter_call should have id in conversation items"
                                            code_interpreter = tool_content.get("code_interpreter")
                                            if code_interpreter:
                                                assert (
                                                    "code" not in code_interpreter
                                                ), "code should NOT be present when content recording is disabled"
                    else:
                        assert False, f"Unexpected event name in list_conversation_items span: {event.name}"

                assert found_code_interpreter_in_items, "Should have found code_interpreter_call in conversation items"

                # Cleanup
                await openai_client.conversations.delete(conversation_id=conversation.id)
                await openai_client.files.delete(file.id)

            finally:
                await project_client.agents.delete_version(agent_name=agent.name, agent_version=agent.version)
